#include "testApp.h"


#define NUM_POINTS            10
#define BOUNCE_FACTOR         0.5
#define ACCELEROMETER_FORCE      0.2
#define RADIUS             30


class Thing {
public:
    ofPoint pos;
    ofPoint vel;
    ofColor col;
    
    int selectedByTouch;
    bool dragged;
    ofPoint lastPos;
    
    void init() {
        pos.set(ofRandomWidth(), ofRandomHeight(), 0);
        vel.set(ofRandomf(), ofRandomf(), 0);
        
        col.r = ofRandom(0, 255);
        col.g = ofRandom(0, 255);
        col.b = ofRandom(0, 255);
        col.a = ofRandom(0, 255);
        
        selectedByTouch = -1;
        dragged = false;
    }
    
    void update() {
         vel.x += ACCELEROMETER_FORCE * ofxAccelerometer.getForce().x * ofRandomuf();
         vel.y += -ACCELEROMETER_FORCE * ofxAccelerometer.getForce().y * ofRandomuf();        // this one is subtracted cos world Y is opposite to opengl Y
        
        // add vel to pos
        pos += vel;
        
        // check boundaries
        if(pos.x < RADIUS) {
            pos.x = RADIUS;
            vel.x *= -BOUNCE_FACTOR;
        } else if(pos.x >= ofGetWidth() - RADIUS) {
            pos.x = ofGetWidth() - RADIUS;
            vel.x *= -BOUNCE_FACTOR;
        }
        
        if(pos.y < RADIUS) {
            pos.y = RADIUS;
            vel.y *= -BOUNCE_FACTOR;
        } else if(pos.y >= ofGetHeight() - RADIUS) {
            pos.y = ofGetHeight() - RADIUS;
            vel.y *= -BOUNCE_FACTOR; 
        }
    }
    
    void draw() {
        ofSetColor(col.r, col.g, col.b, col.a);
        ofCircle(pos.x, pos.y, 30);
    }
    
    void toss() {
        ofPoint dist = pos - lastPos;
        vel = dist;
    }
    
    void moveTo(int x, int y) {
        lastPos = pos;
        pos.set(x, y, 0);
        vel.set(0,0,0);
    }
    
};


Thing things[NUM_POINTS];

//--------------------------------------------------------------
void testApp::setup(){
    printf("setup()\n");
    
    ofBackground(0, 0, 0); // set background to black
    ofSetBackgroundAuto(true); // set background to clear automatically every frame
    ofSetFrameRate(60); // set desired framerate to 60fps
    
    // initialize the accelerometer and multitouch:
    ofxAccelerometer.setup();
    ofxMultiTouch.addListener(this);
    
    // initialize all of the Thing particles
    for(int i=0; i<NUM_POINTS; i++) things[i].init();
}

//--------------------------------------------------------------
void testApp::update(){
    // printf("update()\n");
    for(int i=0; i<NUM_POINTS; i++) things[i].update();
}

//--------------------------------------------------------------
void testApp::draw(){
    // printf("draw()\n");
    for(int i=0; i<NUM_POINTS; i++) things[i].draw();
}


// mouse functions are there for backwards compatibility
// they are simply the first finger to touch the screen
// you can omit the mouse functions and just use touch functions
// or omit touch functions and just use mouse functions if you don't need multitouch
//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
    printf("mouseMoved: %i, %i\n", x, y);  // this will never get called 
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
    printf("mouseDragged: %i, %i %i\n", x, y, button);
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
    printf("mousePressed: %i, %i %i\n", x, y, button);
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
    printf("mouseReleased: %i, %i %i\n", x, y, button);
}
void testApp::mouseReleased(){
}


//--------------------------------------------------------------
void testApp::touchDown(float x, float y, int touchId, ofxMultiTouchCustomData *data) {
	printf("touchDown: %i, %i %i\n", x, y, touchId);
    
    // only select if touched within an object
    bool thingIsTouched = false;
    
    // find the closest and control it.
    int nearestIndex = 0;
    float nearestDistance = 1280; // start pretty large
    for(int i=0;i<NUM_POINTS;i++){
        int thingX = things[i].pos.x;
        int thingY = things[i].pos.y;
        float currentDistance = sqrtf( (thingX - x) * (thingX - x) +
                                      (thingY - y) * (thingY - y));
        if(currentDistance < nearestDistance && currentDistance < RADIUS) {
            thingIsTouched = true;
            nearestDistance = currentDistance;
            nearestIndex = i;
        }
    }
    
    if(thingIsTouched == true) {
        things[nearestIndex].selectedByTouch = touchId; // link this Thing with the current touch
        things[nearestIndex].moveTo(x, y);
    }
}


void testApp::touchMoved(float x, float y, int touchId, ofxMultiTouchCustomData *data) {
	printf("touchMoved: %i, %i %i\n", x, y, touchId);
    for(int i=0;i<NUM_POINTS;i++){
        if(things[i].selectedByTouch == touchId) {
            things[i].moveTo(x, y);  // no need to do this here, doing it touchMoved()
            things[i].dragged = true;
        }
    }
}


void testApp::touchUp(float x, float y, int touchId, ofxMultiTouchCustomData *data) {
	printf("touchUp: %i, %i %i\n", x, y, touchId);
    
    // reset all the touches
    for(int i=0;i<NUM_POINTS;i++){
        //throw if necessary
        if(things[i].selectedByTouch == touchId){ // now looking for specific touches
            if(things[i].dragged == true){
                things[i].toss();
            }
            
            things[i].selectedByTouch = -1; 
            things[i].dragged = false;
        }
    }
}

void testApp::touchDoubleTap(float x, float y, int touchId, ofxMultiTouchCustomData *data) {
	printf("touchDoubleTap: %i, %i %i\n", x, y, touchId);
	ofToggleFullscreen();
}


void testApp::exit(){
    printf("exiting");
}